@@ -86,6 +86,8 @@ extern "C" {
 /* #undef HAVE_CRL */
 #undef HAVE_CRL_IO
 /* #undef HAVE_CRL_IO */
+#undef WOLFSSL_CUSTOM_CURVES
+/* #undef WOLFSSL_CUSTOM_CURVES */
 #undef HAVE_CURVE25519
 #define HAVE_CURVE25519
 #undef HAVE_CURVE448
@@ -368,6 +370,16 @@ extern "C" {
 /* #undef WOLFSSL_WC_KYBER */
 #undef NO_WOLFSSL_STUB
 /* #undef NO_WOLFSSL_STUB */
+#undef HAVE_ECC_SECPR2
+/* #undef HAVE_ECC_SECPR2 */
+#undef HAVE_ECC_SECPR3
+/* #undef HAVE_ECC_SECPR3 */
+#undef HAVE_ECC_BRAINPOOL
+/* #undef HAVE_ECC_BRAINPOOL */
+#undef HAVE_ECC_KOBLITZ
+/* #undef HAVE_ECC_KOBLITZ */
+#undef HAVE_ECC_CDH
+/* #undef HAVE_ECC_CDH */
 
 #ifdef __cplusplus
 }
